/*********
  9bit.se 2018
*********/

// #define MQTT_USE 0

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <Ticker.h>
#include "index_html.h"

MDNSResponder mdns;

// Network credentials
static const char* ssid = "Bandspelare 24";
static const char* password = "CHRISTOPH NIEMANN";

ESP8266WebServer server(80);
Ticker rtc_second_ticker;
Ticker blink_ticker;
Ticker step_1_ticker;
Ticker step_2_ticker;
Ticker step_3_ticker;

static const int gpio13Led = 13;
static const int gpio12Relay = 12;

int toast_sequence_short[3]  = {100, 30, 30}; // {toast, pause, toast}
int toast_sequence_medium[3] = {130, 30, 30};
int toast_sequence_long[3]   = {160, 30, 30};
int toast_sequence_extra[3]  = {0, 0, 30};
int *sequence;

int seconds; // Counting the progress of a toast sequence

String state_name;


// ---- Ticker callbacks: ----

void led_blink()
{
  digitalWrite(gpio13Led, !digitalRead(gpio13Led));  //LED blink
}


void add_second() {
    seconds++;
}


// ---- The toast program. Step durations stored in "toast_sequences" ----
// ---- Some of these functions are also ticker callbacks ----

void begin_toast() 
{
  seconds = 0;
  toast_step_1();
}


void toast_step_1()
{
  digitalWrite(gpio13Led, LOW);  //LED on
  digitalWrite(gpio12Relay, HIGH);
  step_1_ticker.attach(sequence[0], toast_step_2);
  rtc_second_ticker.attach(1, add_second);
}


void toast_step_2() // Pause
{
  digitalWrite(gpio13Led, HIGH);  //LED off
  digitalWrite(gpio12Relay, LOW);
  // blink timer here!!
  step_1_ticker.detach();
  step_2_ticker.attach(sequence[1], toast_step_3);
  blink_ticker.attach(0.5, led_blink);
}


void toast_step_3()
{
  digitalWrite(gpio13Led, LOW);  //LED on
  digitalWrite(gpio12Relay, HIGH);
  step_2_ticker.detach();
  blink_ticker.detach();
  step_3_ticker.attach(sequence[2], toast_step_4);
}


void toast_step_4()
{
  // End of program
  step_3_ticker.detach();
  rtc_second_ticker.detach();
  state_name = "Stop";
  digitalWrite(gpio13Led, HIGH);  //LED off
  digitalWrite(gpio12Relay, LOW);
}


void end_toast()
{
  state_name = "Stop";
  digitalWrite(gpio13Led, HIGH);  //LED off
  digitalWrite(gpio12Relay, LOW);
  step_1_ticker.detach();
  step_2_ticker.detach();
  step_3_ticker.detach();
  blink_ticker.detach();
  rtc_second_ticker.detach();
}


// ---- Handlers initiated by http requests ----

void handleRoot() // Serve Index.html, as defined in Flash memory
{
  String s = FPSTR(index_html); //Read HTML contents from Flash memory
  server.send(200, "text/html", s); //Send web page
}


void handle_progress() // Web page wants to know the width of the progress bar
{
  float percent = 0;
  float len = float(sequence[0] + sequence[1] + sequence[2]);
  
  if(len > 0) percent = 100 * float(seconds + 1) / len;
  if(percent > 100) percent = 100;
  if(state_name == "Stop") percent = 0;  // Reset the progres bar
  
  String progress = String(percent);
  
  server.send(200, "text/plain", progress);
}


void handle_action() // Web page button click selects toast sequence
{
  String s = server.arg("action");
  int server_action_state = s.toInt();

  if(state_name == "Stop") {
        
    switch (server_action_state) {
      case 1:
        {
          sequence = toast_sequence_short;
          begin_toast();
          state_name = "Short";
          break;
        }
      case 2:
        {
          sequence = toast_sequence_medium;
          begin_toast();
          state_name = "Medium";
          break;
        }
      case 3:
        {
          sequence = toast_sequence_long;
          begin_toast();
          state_name = "Long";
          break;
        }
      case 4:
        {
          sequence = toast_sequence_extra;
          begin_toast();
          state_name = "Extra";
          break;
        }
      case 0:
      default:
        {}
    }
  } else {
    if(server_action_state == 0) {
      // Stop
      state_name = "Stop";
      end_toast();
    }
  }
  
  server.send(200, "text/plain", state_name); //Send response to web server
}


// ---- setup, loop ----

void setup(void) {
  state_name = "Stop";
  sequence = toast_sequence_short;

  // preparing GPIOs
  pinMode(gpio12Relay, OUTPUT);
  digitalWrite(gpio12Relay, LOW); // Relay off

  pinMode(gpio13Led, OUTPUT);
  digitalWrite(gpio13Led, LOW);  //LED on

  WiFi.begin(ssid, password);
  Serial.begin(115200);
  delay(1000);

  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  // Create a static ip adress
  IPAddress static_ip(192, 168, 10, 92);
  IPAddress gateway_ip(192, 168, 10, 1);
  IPAddress subnet_mask(255, 255, 255, 0);
  WiFi.config(static_ip, gateway_ip, subnet_mask);

  Serial.println();
  Serial.println();
  Serial.println("Connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  if (mdns.begin("esp8266", WiFi.localIP())) {
    Serial.println("MDNS responder started");
  }

  server.on("/", HTTP_GET, handleRoot);
  server.on("/select_action", handle_action);
  server.on("/progress_update", handle_progress);

  server.begin();
  Serial.println("HTTP server started");

  digitalWrite(gpio13Led, HIGH);  //LED off
}


void loop(void) {
  server.handleClient();
}

